----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    20:48:06 03/22/2011 
-- Design Name: 
-- Module Name:    DDR2_Read_VHDL - Verhalten 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

	--------------------------------------------
	-- Beschreibung :
	--
	-- Die State-Machine liest
	-- einen 32Bit Wert aus dem RAM
	--
	-- die Adresse und Daten werden
	-- von der bergeordneten CONTROL-Unit
	-- gehandelt
	--
	-- hier werden nur (nach Read-Enable)
	-- die Commandos an das RAM
	-- gehandelt
	-- 
	-- solange die Read-Funktion luft,
	-- ist BUSY=1
	--------------------------------------------

entity DDR2_Read_VHDL is

	--------------------------------------------
	-- Port Deklerationen
	--------------------------------------------
	port (
		reset_in : in std_logic;
		clk_in : in std_logic;
		r_command_register : out std_logic_vector(2 downto 0); -- geht ans RAM
		r_cmd_ack : in std_logic; -- kommt vom RAM
		r_burst_done : out std_logic;	-- geht ans RAM
		r_data_valid : in std_logic; -- kommt vom RAM
		read_en : in std_logic; -- kommt vom CONTROL
		read_busy : out std_logic -- geht ans CONTROL
	);
	
end DDR2_Read_VHDL;

architecture Verhalten of DDR2_Read_VHDL is

	--------------------------------------------
	-- Interne Signale
	--------------------------------------------
	
	constant clk_anz : integer := 2; -- warte 3 Clockzyklen (2 bis 0 = 3)
	signal v_counter :  natural range 0 to clk_anz := clk_anz;
	
	type STATE_R_TYPE is (
		R1_NOP,
		R2_READ_CMD,
		R3_WAIT_4_ACK1,
		R4_WAIT_3_CLK,
		R5_WAIT_4_VALID,
		R6_DATA_OK,
		R7_BURST_HI,
		R8_BURST_OK,
		R9_WAIT_4_ACK0
	);
	signal STATE_R : STATE_R_TYPE := R1_NOP;	

begin

	-----------------------------------------
	-- State-Machine zum
	-- lesen eines wortes aus dem RAM
	-----------------------------------------	
	P_Read : process(clk_in,reset_in)
	begin
		if reset_in = '1' then
			-- reset button ist gedrueckt
			STATE_R <= R1_NOP;
			r_command_register <= "000"; -- NOP
			r_burst_done <= '0';							
		elsif falling_edge(clk_in) then
			case STATE_R is
				when R1_NOP =>							
					-- warte auf read enable signal
					r_command_register <= "000"; -- NOP
					v_counter <= clk_anz;
					r_burst_done <= '0';
					if read_en = '1' then
						STATE_R <= R2_READ_CMD;
					end if;	
				when R2_READ_CMD =>					
					-- CMD anlegen
					r_command_register <= "110"; -- READ-CMD
					STATE_R <= R3_WAIT_4_ACK1;
				when R3_WAIT_4_ACK1 =>					
					-- warten auf ACK=1 vom RAM
					if r_cmd_ack = '1' then
						STATE_R <= R4_WAIT_3_CLK;
					end if;	
				when R4_WAIT_3_CLK =>					
					-- warte 3 Clockzyklen
					if v_counter = 0 then
						STATE_R <= R5_WAIT_4_VALID;						
					else
						v_counter <= v_counter - 1;
					end if;	
				when R5_WAIT_4_VALID =>					
					-- warte bis daten gueltig
					if r_data_valid = '1' then
						-- daten sind ok						
					  	STATE_R <= R6_DATA_OK;						
					end if;
				when R6_DATA_OK =>					
					-- burst_done auf HI
					r_burst_done <= '1';
					STATE_R <= R7_BURST_HI;
				when R7_BURST_HI =>
					-- NOP anlegen
					r_command_register <= "000"; -- NOP
					STATE_R <= R8_BURST_OK;
				when R8_BURST_OK =>
					-- burst_done auf Lo
					r_burst_done <= '0';
					STATE_R <= R9_WAIT_4_ACK0;
				when R9_WAIT_4_ACK0 =>	
					-- warten auf ACK=0 vom RAM
					if r_cmd_ack = '0' then
						STATE_R <= R1_NOP;
					end if;					
				when others =>
					NULL;
			end case;
		end if;
	end process P_Read;
	
	-----------------------------------------
	-- Read-Busy erzeugen
	-----------------------------------------		
	read_busy <= '0' when STATE_R=R1_NOP else '1';	

end Verhalten;

